"""
Rest Helper2 is a CLI tool used to make REST calls to APIS
It supports defining apps, envs, and resources which can be exported
as JSON, and imported by another user/machine, so they can easily call
the same REST apis.
"""
__version__ = "0.2"